/*********************************************************************
*                                                                    *          
*                              FLY                                   *
*                           --E#Y#E--				     *
*                         ===MUSCLE===                               *
*                                                                    *
*********************************************************************/
/* Header Files */
#include "uniform.h"

#define MAX 2048

/*-------------------------------------------------------------------------*/

/* Create Shader */
void CreateShader()
{	
	/* Create Shader */
	vertexShader.filename = glCreateShader(GL_VERTEX_SHADER);
	fragmentShader.filename = glCreateShader(GL_FRAGMENT_SHADER);

	/* Open Shader */
	vertexShader.source = open("C/WIN/fly.vert",O_RDONLY);
	fragmentShader.source = open("C/WIN/fly.frag",O_RDONLY);
	read(vertexShader.source, vert, MAX);
	read(fragmentShader.source, frag, MAX);
	/* Send to Source */
	glShaderSource(vertexShader.id, vert[MAX], &tmp, NULL);
	glShaderSource(fragmentShader.id, frag[MAX], &tmp, NULL);
	
	/* Compile vertexShader */
	glCompileShader(vertexShader.id);
	glGetShaderiv(vertexShader.id, GL_COMPILE_STATUS, &vertexShader.result);	
	if(vertexShader.source)
	{
		printf("Could not Compile Shader. %d ", vertexShader.id);
		
	}
	glGetShaderiv(vertexShader.id, GL_INFO_LOG_LENGTH, &vertexShader.info);
	if(vertexShader.info)
	{	
		glGetShaderInfoLog(vertexShader.id, vert[MAX], 
				&vertexShader.info, &vertexShader.log[0]);
		printf("Shader contains Errors! %s ", vertexShader.log);	
	}

	/* Compile fragmentShader */
	glCompileShader(fragmentShader.id);
	glGetShaderiv(fragmentShader.id, GL_COMPILE_STATUS, &fragmentShader.result);	
	if(fragmentShader.source)
	{
		printf("Could not Compile Shader. %d ", fragmentShader.id);
	}
	glGetShaderiv(fragmentShader.id, GL_INFO_LOG_LENGTH, &fragmentShader.info);
	if(fragmentShader.info)
	{	
		glGetShaderInfoLog(fragmentShader.id, frag[MAX], 
				&fragmentShader.info, &fragmentShader.log[0]);
		printf("Shader contains Errors! %s ", fragmentShader.log);
	}
}

/*-------------------------------------------------------------------------*/

/* Create Program */	
void CreateProgram(GLuint shaderProgram)
{	
	/* Create Program */
	shaderProgram = glCreateProgram();

	/* Attach the Shaders to Object */
	glAttachShader(shaderProgram, vertexShader.id);
	glAttachShader(shaderProgram, fragmentShader.id);
	
	/* Link the Program */	
	glLinkProgram(shaderProgram);

	/* Bind Program */
	glUseProgram(shaderProgram);
}

/*-------------------------------------------------------------------------*/

/* Location */
void Location(GLuint shaderProgram, unsigned int index, const char* name)
{
	/* Get Location */
	/*vertexPosition = glGetAttribLocation(shaderProgram, "a_Vertex");*/
	/*colorPosition = glGetAttribLocation(shaderProgram, "a_Color");*/
	/*normalPosition = glGetAttribLocation(shaderProgram, "a_TexCoord");*/
	
	/* Bind */
	location = glGetAttribLocation(shaderProgram, name);
	glBindAttribLocation(shaderProgram, location, name);
}	

/*-------------------------------------------------------------------------*/

/* Uniform */
void Uniform(GLuint shaderProgram, const char* name, const float matrix)
{	
	location = glGetUniformLocation(shaderProgram, name);
	glUniform1f(location, matrix);
}

/*-------------------------------------------------------------------------*/

/* Uniform 4x4 */
void Uniform4x4(GLuint shaderProgram, const char* name , const float* matrix, char transpose)
{
	/* Get Location */
	/*uniform4f(glGetUniformLocation(shaderProgram, "a_Color"), 0, 0, 0, 1);*/
	/*uniform1f(glGetUniformLocation(shaderProgram, "a_TexCoord"), 0);*/
	/*fudgeLocation = glGetUniformLocation(shaderProgram, "fudge_factor");*/
	/*uProjectionMatrixLoc = glGetUniformLocation(shaderProgram, "projection_matrix");*/
	/*uModelViewMatrixLoc = glGetUniformLocation(shaderProgram, "modelView_matrix");*/
	
	/* Get Location */
	transpose = 0; 
	location = glGetUniformLocation(shaderProgram, name);
        glUniformMatrix4fv(location, 1, transpose, matrix);
}

/*-------------------------------------------------------------------------*/

/* Delete Shader */
void DeleteShader(GLuint shaderProgram)
{
	/* Delete the Shaders */
	glDetachShader(shaderProgram, vertexShader.id);	
	glDetachShader(shaderProgram, fragmentShader.id);
	glDeleteShader(vertexShader.id);
	glDeleteShader(fragmentShader.id);
	glDeleteShader(shaderProgram);

	/* File Close */
	close(vertexShader.source);
	close(fragmentShader.source);	
}

/*-------------------------------------------------------------------------*/

